//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Model/Device/BackgroundItems.cpp
//! @brief     Implements BackgroundItem classes
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/Model/Device/BackgroundItems.h"
#include "GUI/Model/Descriptor/DoubleDescriptor.h"
#include "GUI/Support/XML/Serialize.h"
#include "Sim/Background/ConstantBackground.h"
#include "Sim/Background/PoissonBackground.h"

namespace {
namespace Tag {

const QString BackgroundValue("BackgroundValue");

}
} // namespace

ConstantBackgroundItem::ConstantBackgroundItem()
{
    m_backgroundValue.init("Background value", "Constant background value", 0.0, "counts/pixel",
                           3 /* decimals */, RealLimits::nonnegative(), "value");
}

std::unique_ptr<IBackground> ConstantBackgroundItem::createBackground() const
{
    return std::make_unique<ConstantBackground>(m_backgroundValue);
}

void ConstantBackgroundItem::serialize(Streamer& s)
{
    s.assertVersion(0);
    Serialize::rwProperty(s, Tag::BackgroundValue, m_backgroundValue);
}

std::unique_ptr<IBackground> PoissonBackgroundItem::createBackground() const
{
    return std::make_unique<PoissonBackground>();
}

std::unique_ptr<IBackground> NoBackgroundItem::createBackground() const
{
    return {};
}
